from Globals import DTMLFile, InitializeClass
from AccessControl import ClassSecurityInfo, getSecurityManager, SpecialUsers
from OFS.Folder import Folder
from Acquisition import aq_base, aq_parent, Acquired, Implicit
from DateTime import DateTime

def manage_addTrafficQuotaFolder(self, id, title, trafficquota=0, nextTrafficResetDate=DateTime(), trafficResetInterval=30,  manager_role=None, REQUEST=None):
    """ Add a new TrafficQuotaFolder """
    o = TrafficQuotaFolder(id, title, trafficquota, nextTrafficResetDate, trafficResetInterval, manager_role)
    self._setObject(id, o)
    o = self._getOb(id)

    if REQUEST:
        return self.manage_main(self, REQUEST, update_menu=1)

TrafficQuotaFolderAddForm = DTMLFile('dtml/addForm', globals())

TrafficQuotaException = "Exceeded Traffic Quota"

class TrafficQuotaFolder(Folder, Acquired, Implicit):
  meta_type = 'TrafficQuotaFolder'

  manage_editTrafficQuotaForm = DTMLFile('dtml/editForm', globals())

  manage_options=(
      Folder.manage_options+
      (
      {'label': 'Traffic', 'action': 'manage_editTrafficQuotaForm'},
      )
      )

  __ac_permissions__= \
      Folder.__ac_permissions__


  def __init__(self, id, title="", trafficquota=0, nextTrafficResetDate=DateTime(), trafficResetInterval=30, manager_role="Yes", REQUEST=None):
    "Constructor"
    self.id = id
    self._current_traffic = 0
    self.manage_editTrafficQuota(title, trafficquota, nextTrafficResetDate, trafficResetInterval, manager_role, REQUEST)

  def manage_editTrafficQuota(self, title="", trafficquota=0, nextTrafficResetDate=DateTime(), trafficResetInterval=30, manager_role="Yes", REQUEST=None):
    "Edit Properties"
    self._manager_role = 0
    if manager_role:
        self._manager_role = 1
    if not self.authorized_to_edit():
       raise "Unauthorized", \
             "You are not authorized to edit this TrafficQuota folder"
    self.title = title
    self._trafficquota = int(trafficquota)
    try:
      self._nextTrafficResetDate = DateTime(nextTrafficResetDate)
    except:
      self._nextTrafficResetDate = None
    self._trafficResetInterval = trafficResetInterval
    if REQUEST is not None:
      message = "Data Changed"
      REQUEST.RESPONSE.redirect('manage_editTrafficQuotaForm?manage_tabs_message=%s' % message)

  def current_traffic(self):
    "Return currently used Traffic in bytes"
    if (self.getNextTrafficResetDate() is not None and self.getNextTrafficResetDate().isPast()): #Check if Traffic Reset Date is in the Past --> if yes do the Traffic Reset
      self._resetTraffic()
    return int(self._current_traffic)

  def register_traffic(self, bytes):
    "Register that a certain Traffic has happend, eventually raise an error if Quota is exceeded"
    specuser = 0 #do not enforce quota for Emergyency User - borrowed from QuotaFolder
    user=getSecurityManager().getUser()
    if (SpecialUsers.emergency_user and
          aq_base(user) is SpecialUsers.emergency_user):
          LOG('QuotaFolder', INFO,
              "Emergency user detected - not enforcing quota")
          specuser = 1

    if (self.current_traffic() > self.trafficquota() and specuser != 1 and self.trafficquota > 0):
      raise TrafficQuotaException, "You have exceed your Quota for TrafficQuotaFolder at %s. You will have to wait till %s %s when your Trafficcounter is being reset." % (self.absolute_url(), self.getNextTrafficResetDate().ISO(),  self.getNextTrafficResetDate().timezone())
    self._current_traffic = self.current_traffic() + int(bytes)
    get_transaction().commit() #did not seem to work w/o this! Why?

  def trafficquota(self):
    "Return max-traffic"
    return int(self._trafficquota)

  def getNextTrafficResetDate(self):
    "Return next Date of Traffic Reset"
    return self._nextTrafficResetDate

  def getTrafficResetInterval(self):
    "Return Interval of Traffic Reset in Days"
    return self._trafficResetInterval

  def _resetTraffic(self):
    "Reset the Traffic for a certain Folder"
    self._current_traffic = 0 #Reset Traffic
    if (self.getTrafficResetInterval() != 0): #Reset is Interval is given
      self._nextTrafficResetDate = self.getNextTrafficResetDate() + self.getTrafficResetInterval()
    get_transaction().commit() #did not seem to work w/o this! Why?

  def authorized_to_edit(self):
    """
      return true/false if user is authorized to change in current context, also taken from Quotafolder
    """

    security=getSecurityManager()
    security.addContext(self)
    user = security.getUser()
    if not self._manager_role:
      ## XXX use permission?
      return user.allowed(self, ['Manager'])

     ## use aq_base?
    return user.allowed(aq_parent(self), ['Manager'])

  def manager_role(self):
    return self._manager_role

InitializeClass(TrafficQuotaFolder)
